<?php

namespace Teknisa\COC\Controllers;

use Teknisa\GER\Utilities\Environment;
use Teknisa\GER\Services\CrudService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\Remote\FileUploader\Server;

class ApresentacaoController extends \Teknisa\GER\Controllers\TeknisaCrud {

	protected $fileUploader;
	protected $apresentacaoService;
	protected $dataSourceName = "ciclosubgruseapcia_ciclo";
	protected $dataSourceNameToSave  = "ciclosubgruseapcia";

	protected $serverPath  			= "http://192.168.120.158:8081/zeedhi/fileServer/files/";
	protected $pathKey 				= "zeedhiFTP5798e52659203";
	protected $pathImagemIndefinida = "zeedhiFTP5798e52659203\\geral\\57bf19b628f9a.png";

	public function __construct(CrudService $crudService, ParameterBag $parameterBag, Server $fileUploader, $apresentacaoService) {
		$this->fileUploader = $fileUploader;
		$this->apresentacaoService = $apresentacaoService;
    	parent::__construct($crudService, $parameterBag);
	}

	public function save(DTO\Request $request, DTO\Response $response) {
		foreach ($request->getDataSet()->getRows() as $row) {
			if (isset($row['PHOTO_UPLOAD'][0])){
			//IMAGEM CARREGADA
				if (!$row['__is_new']){
				//REMOVE O ANTIGO ARQUIVO DO SERVIDOR
					$oldFfglogosubapcia = $this->apresentacaoService->getOldFfglogosubapcia($row['CDCIA'], $row['CDSUBGRU'], $row['NRAPRESENTACIA'], $row['NRSERVICOGEN'], $row['CDCICLO'], $row['CDCLASSE'], $row['NRORG']);
					if (($oldFfglogosubapcia) && ($oldFfglogosubapcia != $this->pathImagemIndefinida)){
						$this->fileUploader->deleteFile($oldFfglogosubapcia);
					}
				}
				// var_dump($row['PHOTO_UPLOAD'][0]['path']); die;
				$row['FGLOGOSUBAPCIA'] = $row['PHOTO_UPLOAD'][0]['path'];
			} else if ($row['__is_new']) {
			//IMAGEM INDEFINIDA PARA NOVO REGISTRO
				$row['FGLOGOSUBAPCIA'] = $this->pathImagemIndefinida;
			} else if (strpos($row['FGLOGOSUBAPCIA'], $this->pathKey) !== false){
			//IMAGEM INDEFINIDA PARA REGISTRO EXISTENTE (REMOVE O SERVERPATH DA URL NO FIELD DE IMAGEM)
				$row['FGLOGOSUBAPCIA'] = str_replace($this->serverPath, "", $row['FGLOGOSUBAPCIA']);
			} else {
			//IMAGEM INDEFINIDA PARA REGISTRO EXISTENTE (SUBSTITUI PELA URL O BASE64 CARREGADO NO FIELD DE IMAGEM)
				$row['FGLOGOSUBAPCIA'] = $this->apresentacaoService->getOldFfglogosubapcia($row['CDCIA'], $row['CDSUBGRU'], $row['NRAPRESENTACIA'], $row['NRSERVICOGEN'], $row['CDCICLO'], $row['CDCLASSE'], $row['NRORG']);
			}
		}

		parent::save($request, $response);
	}

	public function delete(DTO\Request $request, DTO\Response $response) {
		$row = $request->getRow();
		$oldFfglogosubapcia = $this->apresentacaoService->getOldFfglogosubapcia($row['CDCIA'], $row['CDSUBGRU'], $row['NRAPRESENTACIA'], $row['NRSERVICOGEN'], $row['CDCICLO'], $row['CDCLASSE'], $row['NRORG']);
		if (($oldFfglogosubapcia) && ($oldFfglogosubapcia != $this->pathImagemIndefinida)){
			$this->fileUploader->deleteFile($oldFfglogosubapcia);
		}
		parent::delete($request, $response);
	}

	public function find(DTO\Request\Filter $request, DTO\Response $response) {
		$response2 = new DTO\Response();
		parent::find($request, $response2);
		if ($response2->getError()) {
			$response->setError($response2->getError());
		} else {
			$rows = array();
			foreach($response2->getDataSets()[0]->getRows() as $row) {
				$row['PHOTO_UPLOAD'] = [];
				$row['FGLOGOSUBAPCIA'] = $this->serverPath.$row['FGLOGOSUBAPCIA'];
				$rows[] = $row;
			}
			$response->addDataSet(new DataSet("ciclosubgruseapcia", $rows));
		}
	}
}