<?php

namespace Teknisa\COC\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Filter;
use Zeedhi\Framework\DTO\Response;

class BuscaMenuVigente extends \Teknisa\GER\Controllers\TeknisaCrud {

	protected $dataSourceName = 'manualciam_operador_vigente';
    protected $parameterBagColumns = array("NRORG", "CDFILIAL", "CDCIA", "IDVIGENCIA", "IDSITUAPROPCIA");
    const VIGENTE = 'VIGENTE';
    const NAO_VIGENTE = 'NÃO VIGENTE';

    public function __construct($crudService, $parameterBag, $dataSourceManager) {
    	$this->crudService = $crudService;
    	$this->parameterBag = $parameterBag;
    	$this->dataSourceManager = $dataSourceManager;
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
		$filterCriteria = $request->getFilterCriteria();
        $filterCriteria->setDataSourceName($this->dataSourceName);
        $filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
        $newFilterCriteria = $this->crudService->getPopulatedFilterCriteria($filterCriteria, $this->parameterBagColumns);
		$dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
		$rows = $dataSet->getRows();
		$this->criaColunaVigencia($rows);
		$response->addDataSet(new DataSet($this->dataSourceName, $rows));
	}

	private function criaColunaVigencia(&$rows) {
		foreach ($rows as &$row) {
			if (empty($row['DTINIVGMA']) || empty($row['DTFIMVGMA'])) {
				$row['VIGENCIA'] = '';
			} else {
				$now = new \DateTime();
				$dataFinal = \DateTime::createFromFormat('d/m/Y h:i:s', $row['DTFIMVGMA']);
				if ($dataFinal > $now)
					$row['VIGENCIA'] = self::VIGENTE;
				else
					$row['VIGENCIA'] = self::NAO_VIGENTE;
			}
		}
	}
}