<?php

namespace Teknisa\COC\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Doctrine\DBAL\Connection;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Exception;

class CusteioMenuGridController extends \Teknisa\GER\Controllers\TeknisaCrud{

    const APROVADO = "A";
    const PENDENTE = "P";

	protected $connection;
	protected $environment;
	protected $cdfilial;
	protected $cdcia;
	protected $cdmancia;
	protected $datacusto;
	protected $tipocusto;
	protected $operador;
	protected $nrorg;
	protected $novoCodigo;
    // protected $origem;

	public function __construct($crudService,$parameterBag,$connection,$environment,$entityManager, $novoCodigo) {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->connection = $connection;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->novoCodigo = $novoCodigo;
    }

	public function calcularCustoManualDaCompanhia(DTO\Request $request, DTO\Response $response) {
		$row = $request->getRow();
		$this->cdfilial		= $row['CDFILIAL'];
		$this->cdcia		= $row['CDCIA'];
		$this->cdmancia		= $row['CDMANCIA'];
        if($row['IDCALCULO'] === 'menu') { //calculo menu ou subgrupo
            $this->nrservicociafili		= NULL;
    		$this->cdclasse		        = NULL;
    		$this->cdsubgru		        = NULL;
    		$this->nrseqsubgrusemam		= NULL;
    		$this->cdciclo		        = NULL;
        } else {
            $this->nrservicociafili		= $row['NRSERVICOCIAFILI'];
    		$this->cdclasse		        = $row['CDCLASSE'];
    		$this->cdsubgru		        = $row['CDSUBGRU'];
    		$this->nrseqsubgrusemam		= $row['NRSEQSUBGRUSEMAM'];
    		$this->cdciclo		        = $row['CDCICLO'];
        }
		$this->datacusto	= $row['DATACUSTO'];
		$this->tipocusto	= $row['TIPOCUSTO'];
		$this->cdoperador	= $row['CDOPERADOR'];
		$this->nrorg		= $row['NRORG'];

		$this->calcularCustoDoManualDaCompanhia($response);
		if($row['IDCALCULO'] === 'menu') {
		    $this->tornaCustoPendenteManual($this->cdfilial,$this->cdcia,$this->cdmancia,$this->nrorg);
		} else {
		    $this->tornaCustoPendenteSubgrupo($this->cdfilial, $this->cdcia, $this->cdmancia, $this->nrorg, $this->nrservicociafili, $this->cdclasse, $this->cdsubgru, $this->nrseqsubgrusemam);
		}

	}

	public function calcularCustoDoManualDaCompanhia($response) {
		try {
    		$this->connection->beginTransaction();
    		$procedure = new StoredProcedure($this->connection, 'CUSTO_MANUALCIAM');
            $procedure->addParam(new Param('P_CDFILIAL',        Param::PARAM_INPUT, $this->cdfilial));
            $procedure->addParam(new Param('P_CDCIA',           Param::PARAM_INPUT, $this->cdcia));
            $procedure->addParam(new Param('P_CDMANCIA',        Param::PARAM_INPUT, $this->cdmancia));

            $procedure->addParam(new Param('P_NRSERVICOCIAFILI',    Param::PARAM_INPUT, $this->nrservicociafili));
            $procedure->addParam(new Param('P_CDCLASSE',            Param::PARAM_INPUT, $this->cdclasse));
            $procedure->addParam(new Param('P_CDSUBGRU',            Param::PARAM_INPUT, $this->cdsubgru));
            $procedure->addParam(new Param('P_NRSEQSUBGRUSEMAM',    Param::PARAM_INPUT, $this->nrseqsubgrusemam));
            $procedure->addParam(new Param('P_CDCICLO',             Param::PARAM_INPUT, $this->cdciclo));

            $procedure->addParam(new Param('P_DATACUSTO',       Param::PARAM_INPUT, $this->datacusto));
            $procedure->addParam(new Param('P_TIPOCUSTO',       Param::PARAM_INPUT, $this->tipocusto));
            $procedure->addParam(new Param('P_CDOPERADOR',      Param::PARAM_INPUT, $this->cdoperador));
            $procedure->addParam(new Param('P_NRORG',           Param::PARAM_INPUT, $this->nrorg));
            $procedure->execute();
            $this->connection->commit();
	    }
	    catch (\Exception $e){
			$this->connection->rollback();
            $response->setError(new Error("Error in costing process: ". $e->getMessage(), 500));
        }
	}

	public function PROD_MANUALCIAM($cdfilial,$cdcia,$cdmancia,$datacusto,$tipocusto,$cdoperador,$nrorg, $response, $nrservicoclasse, $dssubgru, $cdciclo, $nritairvision) {
    	try {
    		$this->connection->beginTransaction();
    		$procedure = new StoredProcedure($this->connection, 'PROD_MANUALCIAM');
            $procedure->addParam(new Param('P_CDCIA',            Param::PARAM_INPUT, $cdcia));
            $procedure->addParam(new Param('P_CDFILIAL',         Param::PARAM_INPUT, $cdfilial));
            $procedure->addParam(new Param('P_CDMANCIA',         Param::PARAM_INPUT, $cdmancia));
            $procedure->addParam(new Param('P_NRORG',            Param::PARAM_INPUT, $nrorg));
            $procedure->addParam(new Param('P_DTCUSTO',          Param::PARAM_INPUT, $datacusto));
            $procedure->addParam(new Param('P_IDTIPOCUSTO',      Param::PARAM_INPUT, (int)$tipocusto));
            $procedure->addParam(new Param('P_CDCONTROLE',       Param::PARAM_OUTPUT, null,null,20));
            $procedure->addParam(new Param('P_NRSERVICOCLASSE',  Param::PARAM_INPUT, $nrservicoclasse));
            $procedure->addParam(new Param('P_NRSEQSUBGRUSEMAM', Param::PARAM_INPUT, $dssubgru));
            $procedure->addParam(new Param('P_CDCICLO',          Param::PARAM_INPUT, $cdciclo));
            $procedure->addParam(new Param('P_NRITAIRVISION',    Param::PARAM_INPUT, $nritairvision));
            $result = $procedure->execute();
            $this->connection->commit();
            return $result['P_CDCONTROLE'];
    	}
	    catch (\Exception $e){
			$this->connection->rollback();
            $response->setError(new Error("Error in costing process: ". $e->getMessage(), 500));
        }
	}

	private function createRequest($dataset, $dataSourceName){
        $newDataSet = new DataSet($dataSourceName, $dataset);
        $newRequest = new DTO\Request\DataSet($newDataSet, "", "", "");
        return $newRequest;
    }

	private function updateManual($row,$response) {
		$this->dataSourceName = 'manualciam';
        $newRow = $row;
        $newRow["IDANALIZEATIV"] = "P";
        $newRow["__is_new"] = false;
        $updateManualRequest = $this->createRequest(array($newRow), $this->dataSourceName);
        parent::save($updateManualRequest, $response);
	}

	public function generateProcessWorkflow($row, DTO\Response $response) {
        $this->dataSourceName = 'workflow';
        $newWorkflowManual = array(
            "__is_new" => true,
            "__is_new_code" => true,
            "__new_code_pk" => "WORKFLOW",
            "__table_pk" => "NRWORKFLOW",
            "__new_code_length" => 20,
            "__columns_to_return" => ["NRWORKFLOW"],
            "__cast_number" => true,
            "IDTIPOPERWFLOW" => 1,
            "NRPROCESSOAPROVA" => 4,
            "NRITEMPROCAPROVA" => 1,
            "DSMOTIVOWFLOW" => $row['DSMOTIVOWFLOW'],
            "IDSITWORKFLOW" => 1,
            "NMITEMWFLOW" => $row['NMITEMWFLOW'],
            "CDOPERADOR" => $this->environment->getCdOperador()
        );
        parent::save($this->createRequest(array($newWorkflowManual), $this->dataSourceName), $response);
        $columnsReturn = $this->crudService->getColumnsToReturn();
        if(empty($columnsReturn)){
             throw new \Excpetion("Error! Nenhuma coluna retornada");
        } else {
            $row['COLUMNS_RETURN'] = $columnsReturn;
            return $row;
        }
    }

	public function generateProcessDadoswflow($row, $response) {
	    $this->dataSourceName = 'dadoswflow';
	    $date = \DateTime::createFromFormat('d/m/Y H:i:s', $this->environment->getCurrentTime()->format("d/m/Y H:i:s"));
	    $newDadoswflowManual = array(
            "__is_new" => true,
            "NRWORKFLOW" => $row['COLUMNS_RETURN'][0]['NRWORKFLOW'],
            "NMTABELAWFLOW" => 'SUBGRUSEMAM', //Fixed for Inclusion Process
            "NMCAMPOWFLOW" => 'IDSITCUSTEIO;DTHRAPROVACUSTO;CDOPERAPROVACUSTO',   //Fixed for Inclusion Process
            "DSINFORANTERIOR" => 'P;NULL;NULL',  //Fixed for Inclusion Process
            "DSINFORATUAL" => 'A;'.$date->format('d/m/Y H:i:s').';'.$this->environment->getCdOperador(),   //Fixed for Inclusion Process
            "DSPRIMARYKEY" => 'CDFILIAL='.$row['CDFILIAL'].';CDCIA='.$row['CDCIA'].';CDMANCIA='.$row['CDMANCIA'].';NRORG='.$this->environment->getNrOrg()
        );
        parent::save($this->createRequest(array($newDadoswflowManual), $this->dataSourceName), $response);
	}

	public function generateProcessOperworkflow($row, $operador, $response) {
	    $this->dataSourceName = 'operworkflow';
	    $date = \DateTime::createFromFormat('d/m/Y H:i:s', $this->environment->getCurrentTime()->format("d/m/Y H:i:s"));
        $date->add(new \DateInterval('P'.$operador['NRDIASOPERAPRO'].'D'));
        $newOperworkflowManual = array(
            "__is_new" => true,
            "NRWORKFLOW" => $row['COLUMNS_RETURN'][0]['NRWORKFLOW'],
            "CDOPERADOR" => $operador['CDOPERADOR'],
            "IDSITOPERWFLOW" => 'P',
            "DTLIMITEWFLOW" => $date->format('d/m/Y H:i:s')
        );
        parent::save($this->createRequest(array($newOperworkflowManual), $this->dataSourceName), $response);
	}

	public function generateWorkflow(DTO\Request\Row $request, DTO\Response $response) {
		try {
			$this->entityManager->getConnection()->beginTransaction();
			$row = $request->getRow();

		    $recipes = $this->getRecSubSeMam($row['CDFILIAL'], $row['CDCIA'], $row['CDMANCIA'], $this->environment->getNrOrg());
            $this->insertHistCustoRecMam($recipes);

            $products = $this->getProdSubSeMam($row['CDFILIAL'], $row['CDCIA'], $row['CDMANCIA'], $this->environment->getNrOrg());
            $this->insertHistCustoProdMam($products);

			$this->updateManual($row,$response);
			$this->generateProcessWorkflow($row,$response);
			$this->generateProcessDadoswflow($row, $response);
			foreach($row['OPERADORES'] as $operador) {
				//para cada operador, criar dadoswflow e operwokflow
				$this->generateProcessOperworkflow($row, $operador, $response);
			}
			//se todos os saves para o workflow estiverem corretos commita
			$this->entityManager->getConnection()->commit();
		}catch (\Exception $e){
			//se tiver algum problema volta para os valores anterios e não salva o workflow pela metade
			$this->entityManager->getConnection()->rollback();
            $response->setError(new Error("Error in WORKFLOW process. ". $e->getMessage(), 500));
        }
	}

	private function insertHistCustoProdMam($produtos) {

        for($i = 0; $i < count($produtos); $i++) {

            $produto = $produtos[$i];

            $nrseqhistcustoprodmam = $this->novoCodigo->getCodeByTableName('HISTCUSTOPRODMAM', $this->crudService->getEnvironment()->getNrOrg(), 1, 10, true);
            $nrorg = (Int)$produto['NRORG'];
            $cdfilial = $produto['CDFILIAL'];
            $cdcia = $produto['CDCIA'];
            $cdmancia = $produto['CDMANCIA'];
            $nrservicociafili = (Int)$produto['NRSERVICOCIAFILI'];
            $cdclasse = $produto['CDCLASSE'];
            $cdsubgru = $produto['CDSUBGRU'];
            $nrseqsubgrusemam = (Int)$produto['NRSEQSUBGRUSEMAM'];
            $cdciclo = $produto['CDCICLO'];
            $cdproduto = $produto['CDPRODUTO'];
            $vrcusprsubsemam = $produto['VRCUSPRSUBSEMAM'];

    	    $sql = "INSERT INTO HISTCUSTOPRODMAM (CDFILIAL, CDCIA, CDMANCIA, NRSERVICOCIAFILI, CDCLASSE, CDSUBGRU, NRSEQSUBGRUSEMAM, CDCICLO, NRSEQHISTCUSTOPRODMAM, CDPRODUTO, VRCUSPRSUBSEMAM, NRORG, IDATIVO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU) VALUES (:CDFILIAL, :CDCIA, :CDMANCIA, :NRSERVICOCIAFILI, :CDCLASSE, :CDSUBGRU, :NRSEQSUBGRUSEMAM, :CDCICLO, :NRSEQHISTCUSTOPRODMAM, :CDPRODUTO, :VRCUSPRSUBSEMAM, :NRORG, :IDATIVO, :DTINCLUSAO, :DTULTATU, :NRORGINCLUSAO, :CDOPERINCLUSAO, :NRORGULTATU, :CDOPERULTATU)";

    	    $stmt = $this->connection->prepare($sql);

    	    $stmt->execute(
    	        array(
                    "CDOPERULTATU" => $this->environment->getCdOperador(),
                    "NRORGULTATU" => $this->environment->getNrOrg(),
                    "NRORGINCLUSAO" => $this->environment->getNrOrg(),
                    "CDOPERINCLUSAO" => $this->environment->getCdOperador(),
                    "NRORGINCLUSAO" => $this->environment->getNrOrg(),
                    "IDATIVO" => "S",
                    "NRORG" => $nrorg,
                    "NRSEQHISTCUSTOPRODMAM" => $nrseqhistcustoprodmam,
                    "CDFILIAL" => $cdfilial,
                    "CDCIA" => $cdcia,
                    "CDMANCIA" => $cdmancia,
                    "NRSERVICOCIAFILI" => $nrservicociafili,
                    "CDCLASSE" => $cdclasse,
                    "CDSUBGRU" => $cdsubgru,
                    "NRSEQSUBGRUSEMAM" => $nrseqsubgrusemam,
                    "CDCICLO" => $cdciclo,
                    "CDPRODUTO" => $cdproduto,
                    "VRCUSPRSUBSEMAM" => $vrcusprsubsemam,
                    "DTINCLUSAO" => $this->environment->getCurrentTime()->format("d/m/Y H:i:s"),
                    "DTULTATU" => $this->environment->getCurrentTime()->format("d/m/Y H:i:s")

                )
            );

        }

	}

	private function insertHistCustoRecMam($receitas) {

        for($i = 0; $i < count($receitas); $i++) {

            $receita = $receitas[$i];

            $nrseqhistcustorecmam = $this->novoCodigo->getCodeByTableName('HISTCUSTORECMAM', $this->crudService->getEnvironment()->getNrOrg(), 1, 10, true);
            $nrorg = $receita['NRORG'];
            $cdfilial = $receita['CDFILIAL'];
            $cdcia = $receita['CDCIA'];
            $cdmancia = $receita['CDMANCIA'];
            $nrservicociafili = $receita['NRSERVICOCIAFILI'];
            $cdclasse = $receita['CDCLASSE'];
            $cdsubgru = $receita['CDSUBGRU'];
            $nrseqsubgrusemam = $receita['NRSEQSUBGRUSEMAM'];
            $cdciclo = $receita['CDCICLO'];
            $cdreceita = $receita['CDRECEITA'];
            $vrcusresubsemam = $receita['VRCUSRESUBSEMAM'];


    	    $sql = "INSERT INTO HISTCUSTORECMAM (NRORG, NRSEQHISTCUSTORECMAM, CDFILIAL, CDCIA, CDMANCIA, NRSERVICOCIAFILI, CDCLASSE, CDSUBGRU, NRSEQSUBGRUSEMAM, CDCICLO, CDRECEITA, VRCUSRESUBSEMAM, DTINCLUSAO, DTULTATU) VALUES (:NRORG, :NRSEQHISTCUSTORECMAM, :CDFILIAL, :CDCIA, :CDMANCIA, :NRSERVICOCIAFILI, :CDCLASSE, :CDSUBGRU, :NRSEQSUBGRUSEMAM, :CDCICLO, :CDRECEITA, :VRCUSRESUBSEMAM, :DTINCLUSAO, :DTULTATU)";

    	    $stmt = $this->connection->prepare($sql);

    	    $stmt->execute(
    	        array(
                    "NRORG" => $nrorg,
                    "NRSEQHISTCUSTORECMAM" => $nrseqhistcustorecmam,
                    "CDFILIAL" => $cdfilial ,
                    "CDCIA" => $cdcia ,
                    "CDMANCIA" => $cdmancia ,
                    "NRSERVICOCIAFILI" => $nrservicociafili ,
                    "CDCLASSE" => $cdclasse ,
                    "CDSUBGRU" => $cdsubgru ,
                    "NRSEQSUBGRUSEMAM" => $nrseqsubgrusemam ,
                    "CDCICLO" => $cdciclo ,
                    "CDRECEITA" => $cdreceita ,
                    "VRCUSRESUBSEMAM" => $vrcusresubsemam,
                    "DTINCLUSAO" => $this->environment->getCurrentTime()->format("d/m/Y H:i:s"),
                    "DTULTATU" => $this->environment->getCurrentTime()->format("d/m/Y H:i:s")

                )
            );

        }

	}

	private function updateSubGruSeMam($cdfilial,$cdcia,$cdmancia,$nrorg) {

	    $sql = "UPDATE SUBGRUSEMAM SET DTHRAPROVACUSTO = :DTHRAPROVACUSTO, CDOPERAPROVACUSTO = :CDOPERAPROVACUSTO, IDSITCUSTEIO = :IDSITCUSTEIO
	    WHERE CDFILIAL = :CDFILIAL AND CDCIA = :CDCIA AND CDMANCIA = :CDMANCIA AND NRORG = :NRORG ";

	    $stmt = $this->connection->prepare($sql);

	    $stmt->execute(
	        array(
                "CDFILIAL" => $cdfilial,
                "CDCIA" => $cdcia,
                "CDMANCIA" => $cdmancia,
                "NRORG" => $nrorg,
                "DTHRAPROVACUSTO" => $this->environment->getCurrentTime()->format("d/m/Y H:i:s"),
                "CDOPERAPROVACUSTO" => $this->environment->getCdOperador(),
                "IDSITCUSTEIO" => self::APROVADO
            )
        );

	}

	private function tornaCustoPendenteManual($cdfilial,$cdcia,$cdmancia,$nrorg) {

	    $sql = "UPDATE SUBGRUSEMAM SET IDSITCUSTEIO = :IDSITCUSTEIO WHERE CDFILIAL = :CDFILIAL AND CDCIA = :CDCIA AND CDMANCIA = :CDMANCIA AND NRORG = :NRORG";

	    $stmt = $this->connection->prepare($sql);

	    $stmt->execute(
	        array(
                "CDFILIAL" => $cdfilial,
                "CDCIA" => $cdcia,
                "CDMANCIA" => $cdmancia,
                "NRORG" => $nrorg,
                "IDSITCUSTEIO" => self::PENDENTE
            )
        );
	}

	private function tornaCustoPendenteSubgrupo($cdfilial, $cdcia, $cdmancia, $nrorg, $nrservicociafili, $cdclasse, $cdsubgru, $nrseqsubgrusemam) {

	    $sql = "UPDATE SUBGRUSEMAM SET IDSITCUSTEIO = :IDSITCUSTEIO
	    WHERE CDFILIAL = :CDFILIAL AND CDCIA = :CDCIA AND CDMANCIA = :CDMANCIA AND NRORG = :NRORG AND NRSERVICOCIAFILI = :NRSERVICOCIAFILI AND CDCLASSE = :CDCLASSE AND CDSUBGRU = :CDSUBGRU AND NRSEQSUBGRUSEMAM = :NRSEQSUBGRUSEMAM";

	    $stmt = $this->connection->prepare($sql);

	    $stmt->execute(
	        array(
                "CDFILIAL" => $cdfilial,
                "CDCIA" => $cdcia,
                "CDMANCIA" => $cdmancia,
                "NRORG" => $nrorg,
                "NRSERVICOCIAFILI" => $nrservicociafili,
                "CDCLASSE" => $cdclasse,
                "CDSUBGRU" => $cdsubgru,
                "NRSEQSUBGRUSEMAM" => $nrseqsubgrusemam,
                "IDSITCUSTEIO" => self::PENDENTE
            )
        );

	}

	private function getRecSubSeMam($cdfilial, $cdcia, $cdmancia, $nrorg) {

	    $sql = "SELECT NRORG, CDFILIAL, CDCIA, CDMANCIA, NRSERVICOCIAFILI, CDCLASSE, CDSUBGRU, NRSEQSUBGRUSEMAM, CDCICLO, CDRECEITA, VRCUSRESUBSEMAM FROM RECSUBSEMAM WHERE CDFILIAL = :CDFILIAL AND CDCIA = :CDCIA AND CDMANCIA = :CDMANCIA AND NRORG = :NRORG ";

	    $stmt = $this->connection->prepare($sql);

	    $stmt->execute(
	        array(
                "CDFILIAL" => $cdfilial,
                "CDCIA" => $cdcia,
                "CDMANCIA" => $cdmancia,
                "NRORG" => $nrorg
            )
        );

        return $stmt->fetchAll(\PDO::FETCH_ASSOC);

	}

	private function getProdSubSeMam($cdfilial, $cdcia, $cdmancia, $nrorg) {

	    $sql = "SELECT NRORG, CDFILIAL, CDCIA, CDMANCIA, NRSERVICOCIAFILI, CDCLASSE, CDSUBGRU, NRSEQSUBGRUSEMAM, CDCICLO, CDPRODUTO, VRCUSPRSUBSEMAM FROM PRODSUBSEMAM WHERE CDFILIAL = :CDFILIAL AND CDCIA = :CDCIA AND CDMANCIA = :CDMANCIA AND NRORG = :NRORG";

	    $stmt = $this->connection->prepare($sql);

	    $stmt->execute(
	        array(
                "CDFILIAL" => $cdfilial,
                "CDCIA" => $cdcia,
                "CDMANCIA" => $cdmancia,
                "NRORG" => $nrorg
            )
        );

        return $stmt->fetchAll(\PDO::FETCH_ASSOC);

	}

	public function aprovacaoDoCusteio(DTO\Request $request, DTO\Response $response) {

	    $row = $request->getRow();

        $cdfilial = $row['CDFILIAL'];
        $cdcia = $row['CDCIA'];
        $cdmancia = $row['CDMANCIA'];
        $nrorg = $this->environment->getNrOrg();

        try{

            $this->updateSubGruSeMam($cdfilial, $cdcia, $cdmancia, $nrorg);

            $recsubsemam = $this->getRecSubSeMam($cdfilial, $cdcia, $cdmancia, $nrorg);
            $this->insertHistCustoRecMam($recsubsemam);

            $prodsubsemam = $this->getProdSubSeMam($cdfilial, $cdcia, $cdmancia, $nrorg);
            $this->insertHistCustoProdMam($prodsubsemam);

            // $response->addDataset(new DataSet('result', []));

        }catch(\Exception $e){
            $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
        }

	}

}