<?php

namespace Teknisa\COC\Controllers;

use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DTO\Response\Error;
use \Zeedhi\Framework\DataSource\DataSet;
use \Zeedhi\Framework\DataSource\FilterCriteria;
use \Zeedhi\Framework\DataSource\Manager;
use \Zeedhi\Framework\DataSource\ParameterBag;
use \Teknisa\GER\Services\CrudService;
use \Teknisa\GER\Utilities\Messages;
// use Teknisa\COC\Controllers;
use Zeedhi\Framework\ORM\DateTime;

class VariacaoReceita extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $dataSourceName = 'variacaorec';
    protected $newCode;
    protected $modpreprecmTabela;
    protected $environment;
    protected $parameterBag;
    protected $connection;

    public function __construct($crudService, $parameterBag, $dataSourceManager, $environment, $modpreprecmTabela, $newCode, WorkflowReceitaController $WorkflowReceitaController, $connection) {
        $this->dataSourceManager = $dataSourceManager;
        $this->environment=$environment;
        $this->modpreprecmTabela = $modpreprecmTabela;
        $this->newCode=$newCode;
        $this->WorkflowReceitaController = $WorkflowReceitaController;
        $this->connection = $connection;
        parent::__construct($crudService, $parameterBag);
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response){

        $filtercriteria = $request->getFilterCriteria();
        $filtercriteria->addCondition("NRORG", "=", $this->environment->getNrOrgTrab());
        parent::find($request, $response);

    }
    
    private function getRowsFromRequest($request){
        $rows = array();
        if ($request instanceof DTO\Request\Row) {
            $rows = array($request->getRow());
        }
        if ($request instanceof DTO\Request\DataSet) {
            $rows = $request->getDataSet()->getRows();
        }
        return $rows;
    }
    
    public function save(DTO\Request $request, DTO\Response $response) {
        $dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
        if($dataSourceName === 'variacaorec') {
            try{
                $this->connection->beginTransaction();
                $datasetToSave = array();

                $cdreceitaToFiliRec = null;
                $nrvariacaorecToFiliRec = null;
                
                $rows = $this->getRowsFromRequest($request);
                foreach($rows as $row) {
                    if($request->getMethod() !== ""){
                        $ultimaVariacao = $this->retornaUltimaVariacaorec($request, $response);
                        $row["NRVARIACAOREC"] = intval($ultimaVariacao) + 1;
                    }
                    $cdreceitaToFiliRec = ($cdreceitaToFiliRec !== null) ? $cdreceitaToFiliRec : $row['CDRECEITA'];
                    $nrvariacaorecToFiliRec = ($nrvariacaorecToFiliRec !== null) ? $nrvariacaorecToFiliRec : $row['NRVARIACAOREC'];
                    $row['CDOPERADOR'] = $row['CDOPERADOR_TO_SAVE'];
                    
                    array_push($datasetToSave, $row);
                }
                
                $newRequest = $this->createRequest($datasetToSave, 'variacaorec');
                $rowsToSave = $this->crudService->getRowsToSave($newRequest, 'variacaorec');
                
                $rows = $rowsToSave;
                $this->crudService->save('variacaorec', $rowsToSave);
                $this->addFiliRec($cdreceitaToFiliRec, $nrvariacaorecToFiliRec);
                
                $this->verificaAlteracoes($request,$response,$rows);
                $this->connection->commit();
            } catch(\Exception $e){
                $response->setError(new Error($e->getMessage(), 500));
                $this->connection->rollback();
            }
        }
        else {
            $rows = $this->crudService->getRowsToSave($request, $dataSourceName);
            $this->crudService->save($dataSourceName, $rows);
            $this->verificaAlteracoes($request,$response,$rows);
        }

    }

    private function retornaUltimaVariacaorec($request, $response){
        $maxVariacaorecDts = 'variacaorec_max';
        $row = $request->getMethod() === "" ? $request->getDataset()->getRows()[0] : $request->getRow();
        $conditions = array(
            array(
                'columnName' => 'CDRECEITA',
                'operator' => '=',
                'value' => $row['CDRECEITA']
            ),
            array(
                'columnName' => 'NRORG',
                'operator' => '=',
                'value' =>  $this->environment->getNrOrgTrab()
            )
        );
        $filtercriteria = new FilterCriteria($maxVariacaorecDts, $conditions);
        $newRequest = new DTO\Request\Filter($filtercriteria, null, null, null);
        try {
            $filterCriteria = $newRequest->getFilterCriteria();
            $filterCriteria->setDataSourceName($maxVariacaorecDts);
            $filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
            $newFilterCriteria = $this->crudService->getPopulatedFilterCriteria($filterCriteria, $this->parameterBagColumns);
            $this->crudService->find($response, $newFilterCriteria);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
        $row = $response->getDataSets()[0]->getRows()[0];
        return $row["MAXVARIACAOREC"];
    }

    private function addFiliRec($cdreceita, $nrvariacaorec){
        $sql = "INSERT INTO FILIRECEITA (CDFILIAL, CDRECEITA, NRVARIACAOREC, NRORG, IDATIVO, DTINCLUSAO,
                DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU)
                SELECT CDFILIAL, :CDRECEITA, :NRVARIACAOREC, :NRORG, 'S', SYSDATE,
                SYSDATE, :NRORG, :CDOPERINCLUSAO, :NRORG, :CDOPERULTATU FROM FILIAL
                WHERE CDFILIAL NOT IN (SELECT CDFILIAL FROM FILIRECEITA
                WHERE CDRECEITA = :CDRECEITA
                AND NRORG = :NRORG)";
        $params = array(
            "CDRECEITA" => $cdreceita,
            "NRVARIACAOREC" => $nrvariacaorec,
            "NRORG" => $this->environment->getNrOrgTrab(),
            "CDOPERINCLUSAO" => $this->environment->getCdOperador(),
            "CDOPERULTATU" => $this->environment->getCdOperador()
        );
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
    }

    public function getDadoswFlow(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = 'workflow_dadoswflow';
        $this->parameterBagColumns = array("DSPRIMARYKEY", "NMTABELAWFLOW", "NRORG");
        parent::find($request, $response);
    }
    private function updateVariacaorec($row, $response){
        $this->dataSourceName = 'variacaorec';
        if (!empty($row['UPDATEINTABLE']))
            $this->dataSourceName = $row['UPDATEINTABLE'];
        $newRow = $row;
        $newRow["IDANALIZEATIV"] = "P";
        parent::save($this->createRequest(array($newRow), $this->dataSourceName), $response);
    }
    private function saveProdOrSubreceita($row, $response){
        $this->dataSourceName = $row['UPDATEINTABLE'];
        $newRow = $row;
        $newRow["IDANALIZEATIV"] = "P";
        parent::save($this->createRequest(array($newRow), $this->dataSourceName), $response);
        $columnsReturn = $this->returnColumns();
        if(empty($columnsReturn)){
             $response->setError(new Error("Error! Nenhuma coluna retornada". $e->getMessage(), 500));
        } else {
            return $columnsReturn;
        }
    }

    public function returnColumns(){
        return $this->crudService->getColumnsToReturn();
    }
    private function createRequest($row, $dataSourceName){
        $newDataSet = new DataSet($dataSourceName, $row);
        $newRequest = new DTO\Request\DataSet($newDataSet, "", "", "");
        return $newRequest;
    }

    public function generateWflow(DTO\Request\Row $request, DTO\Response $response){
        //NRPROCESSOAPROVA = '1' -> Processo Aprovação Receita fixed
        try {
            $row = $request->getRow();
            //Recebe todos os operadores que deverão aprovar ou não o processo em questão
            $operEnvolvidos = $this->WorkflowReceitaController->getOperatorsInvolved($row, $response);
            if(!empty($operEnvolvidos)){
                //Altera campo de controle de WorkFlow na Variação - IDANALIZEATIV
                $this->updateVariacaorec($row, $response);
                //Gerar Processo de WorkFlow com o Operador logado
                $workflowGerado = $this->WorkflowReceitaController->newProcessWorkflow($row, $request, $response);
                //Cria solicitação para todos os Operadores Responsáveis
                $this->WorkflowReceitaController->operWorkflow($operEnvolvidos, $workflowGerado, $response);
                //Solicita alteração na tabela de VARIACAOREC
                if(empty($row['NMTABELAWFLOW']))
                    $row["NMTABELAWFLOW"] = 'VARIACAOREC';
                $this->WorkflowReceitaController->saveDadoswflow($row, $workflowGerado, $response);
            } else {
                $response->setError(new Error("Error in WORKFLOW process. Nenhum operador encontrado para o processo de WORKFLOW" ,500));
            }
        }catch (\Exception $e){
            $response->setError(new Error("Error in WORKFLOW process. ". $e->getMessage(), 500));
        }

    }
    public function generateWflowEdit(DTO\Request $request, DTO\Response $response){
        try{
            //IDTIPOPERWFLOW -- 1 -> UPDATE
            //               -- 2 -> INSERT
            //               -- 3 -> DELETE
            $row = $request->getDataSet()->getRows();
            $oldRow = $row[0];
            $validateProcess = $row[1];
            $newRow = $row[2];
            //Progress
            $oldRow['NRPROCESSOAPROVA'] = 1;
            foreach ($validateProcess as $field) {
                if(!empty($field)){
                    $oldRow['NRITEMPROCAPROVA'] = $field[array_keys($field)[0]];
                    $operator = $this->WorkflowReceitaController->getOperatorsInvolved($oldRow, $response);
                    if(!empty($operator)) {
                        //Altera campo de controle de WorkFlow na Receitam - IDANALIZEATIV
                        $this->updateVariacaorec($oldRow, $response);
                        //Gerar Processo de WorkFlow com o Operador logado
                        $oldRow['IDTIPOPERWFLOW'] = 1;
                        $oldRow['DSMOTIVOACAO'] = $newRow['DSMOTIVOACAO'];
                        $oldRow['IDSITWORKFLOW'] = 1;
                        $workflowGerado = $this->WorkflowReceitaController->newProcessWorkflow($oldRow, $request, $response);
                        //Cria solicitação para todos os Operadores Responsáveis
                        $this->WorkflowReceitaController->operWorkflow($operator, $workflowGerado, $response);
                        //Solicita alteração na tabela de RECEITAM para as alterações feitas
                        $newRow['NMCAMPOWFLOW'] = array_keys($field)[0];
                        $oldRow['DSINFORANTERIOR'] = $oldRow[array_keys($field)[0]];
                        $newRow['DSINFORATUAL'] = $newRow[array_keys($field)[0]];
                        $newRow["NMTABELAWFLOW"] = 'VARIACAOREC';
                        $this->WorkflowReceitaController->updateDadoswflow($oldRow, $newRow, $workflowGerado, $response);
                    }else {
                        $itemprocaprova = $this->WorkflowReceitaController->getItemprocaprova($oldRow, $response);
                        $response->setError(new Error("Error in WORKFLOW process. Nenhum operador encontrado para o processo de '".$itemprocaprova[0]['NMITEMPROCAPROVA']."'" ,500));
                    }
                }
            }
        } catch (\Exception $e){
            $response->setError(new Error("Error in WORKFLOW process. ". $e->getMessage(), 500));
        }
    }
    private function saveNewVariacao($row, DTO\Response $response){
        $this->dataSourceName = 'variacaorec';
        $this->save($this->createRequest(array($row), $this->dataSourceName), $response);
        $columnsReturn = $this->returnColumns();
        if(empty($columnsReturn)){
             //$response->setError(new Error("Error! Nenhuma coluna retornada", 500));
        } else {
            return $columnsReturn;
        }
        return $columnsReturn;
    }
    private function saveNewComposicao($dataset, $nrvariacaorec, $dataSourceName, $response){
        $this->dataSourceName = $dataSourceName;
        $newDataSet = array();
        foreach ($dataset as $row) {
            $row['NRVARIACAOREC'] = $nrvariacaorec;
            $row['__is_new'] = true;
            array_push($newDataSet, $row);
        }
        $newRequest = $this->createRequest($newDataSet, $this->dataSourceName);
        $this->save($newRequest, $response);
    }
    private function findComposicao($row, $dataSourceName, DTO\Response $response){
        $this->dataSourceName = $dataSourceName;
        $this->parameterBagColumns = array();
        $condition = array(
            array(
                "columnName" => "CDRECEITA",
                "operator" => "=",
                "value" => $row["CDRECEITA"]
            ),
            array(
                "columnName" => "NRVARIACAOREC",
                "operator" => "=",
                "value" => $row["NRVARIACAOREC"]
            ),
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $row["NRORG"]
            )
        );
        $filterCriteria= new FilterCriteria($this->dataSourceName, $condition);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        $dataset = $this->dataSourceManager->findBy($newFilterCriteria);
        return $dataset->getRows();
    }
    public function duplicaVariacao(DTO\Request\Row $request, DTO\Response $response){
        try {
            $row = $request->getRow();
            $ultimaVariacao = $this->retornaUltimaVariacaorec($request, $response);
            $newVariacao = array(
                "CDRECEITA" => $row['CDRECEITA'],
                "CDOPERADOR" => $row['CDOPERADOR'],
                "CDOPERADOR_TO_SAVE" => $row['CDOPERADOR_TO_SAVE'],
                "NMVARIACAOREC" => "",
                "NRVARIACAOREC" => $ultimaVariacao + 1,
                "QTRENDIMENTOREC" => $row['QTRENDIMENTOREC'],
                "NRTEMPOPREP" => 0,
                "NRTEMPOPREPARO" => floatval($row['NRTEMPOPREPARO'] ? $row['NRTEMPOPREPARO'] : 0),
                "IDATIVO" => $row['IDATIVO'] ? $row['IDATIVO'] :'S',
                "DTCUSTOVARIACAO" => null,
                "IDTIPOCUSTOVAR" => null,
                "IDANALIZEATIV" => $row['IDANALIZEATIV'] ? $row['IDANALIZEATIV'] : null,
                "__columns_to_return"  => ["NRVARIACAOREC"],
                "__is_new" => true
            );
            $this->saveNewVariacao($newVariacao, $response);
            $composicao_produtos = $this->findComposicao($row, 'proreceitam', $response);
            $composicao_subreceitas = $this->findComposicao($row, 'subreceitam', $response);
            if(!empty($composicao_produtos))
                $this->saveNewComposicao($composicao_produtos, $newVariacao['NRVARIACAOREC'], 'proreceitam', $response);
            if(!empty($composicao_subreceitas))
                $this->saveNewComposicao($composicao_subreceitas, $newVariacao['NRVARIACAOREC'], 'subreceitam', $response);
            $mododepreparo = $this->findComposicao($row, 'modpreprecm', $response);
            if(!empty($mododepreparo))
                $this->saveNewComposicao($mododepreparo, $newVariacao['NRVARIACAOREC'], 'modpreprecm', $response);
        }catch(\Exception $e){
            $response->setError(new Error("Error in DUPLICAR process. ". $e->getMessage(), 500));
        }
    }
    
    public function solicitaAlteracaoUnidadeVariacao(DTO\Request $request, DTO\Response $response){
        $operadoresEnvolvidos = $this->operadoresEnvolvidoUnidade('1', '34');
        if(!empty($operadoresEnvolvidos)){
            $nrworkflow = $this->newWorkflowDeleteUnidade($request);
            $this->novoDadosWorkFlow($request, $nrworkflow);
            $this->vinculaOperadoresWorkflow($nrworkflow, $operadoresEnvolvidos);
        }else {
            $response->setError(new Error("Error in WORKFLOW process. Nenhum operador encontrado para o processo de WORKFLOW" ,500));
        }
    }
    
    private function newWorkflowDeleteUnidade($request){
        $nmitemwflow = 'Receita: '.$request->getDataSet()->getRows()[0]["unidadesDelete"][0]["DSRECEITA"].' Variação: '.$request->getDataSet()->getRows()[0]["unidadesDelete"][0]["NRVARIACAOREC"];
        $row = array(
            "NRWORKFLOW"       => $this->newCode->getCodeByTableName('WORKFLOW', $this->crudService->getEnvironment()->getNrOrg(), 1, 10, true),
            "CDOPERADOR"       => $this->environment->getCdOperador(),
            "NRPROCESSOAPROVA" => 1,
            "NRITEMPROCAPROVA" => 34,
            "NMITEMWFLOW"      => $nmitemwflow,
            "DSMOTIVOWFLOW"    => $request->getDataSet()->getRows()[2]['row']['DSMOTIVOACAO'],
            "IDTIPOPERWFLOW"   => '3',
            "IDSITWORKFLOW"    => '1',
            "__is_new"         => true
          );
        $newDataSet = new DataSet('workflow', array($row)); 
        $newRequest = new DTO\Request\DataSet($newDataSet, "", "", ""); 
        $rowsComDefaults = $this->crudService->getRowsToSave($newRequest, 'workflow');
        $this->crudService->save('workflow', $rowsComDefaults);
        return $row['NRWORKFLOW'];
    }
    
    private function novoDadosWorkFlow($request, $nrworkflow){
        $unidadesDelete = $request->getDataSet()->getRows()[0]['unidadesDelete'];
        $unidadesSalvar = $request->getDataSet()->getRows()[1]['unidadesSalvar'];
        $unidades = array();
        for($i=0; $i<sizeof($unidadesDelete); $i++){
            array_push($unidades, "'".$unidadesDelete[$i]['CDFILIAL']."'");
        }
        $unidades = implode($unidades, ',');
        $row = array(
            'NRWORKFLOW'      => $nrworkflow,
            'NMTABELAWFLOW'   => 'FILIRECEITA',
            'NMCAMPOWFLOW'    => 'NRVARIACAOREC',
            'DSINFORANTERIOR' => $unidadesDelete[0]['NRVARIACAOREC'],
            'DSINFORATUAL'    => $unidadesSalvar[0]['NRVARIACAOREC'],
            'DSPRIMARYKEY'    => "CDRECEITA='".$unidadesDelete[0]['CDRECEITA']."';CDFILIAL IN (".$unidades.")",
            '__is_new'        => true
        );
        $newDataSet = new DataSet('dadoswflow', array($row)); 
        $newRequest = new DTO\Request\DataSet($newDataSet, "", "", ""); 
        $rowsComDefaults = $this->crudService->getRowsToSave($newRequest, 'dadoswflow');
        
        $this->crudService->save('dadoswflow', $rowsComDefaults);
    }
    
    private function vinculaOperadoresWorkflow($nrworkflow, $operadores){
        
        $dataSet = array();
        foreach ($operadores as $operador) {
            $date = \DateTime::createFromFormat('d/m/Y H:i:s', $this->environment->getCurrentTime()->format("d/m/Y H:i:s"));
            $date->add(new \DateInterval('P'.$operador['NRDIASOPERAPRO'].'D'));
            $row = array(
                'NRWORKFLOW'      => $nrworkflow,
                'CDOPERADOR'      => $operador['CDOPERADOR'],
                'IDSITOPERWFLOW'  => 'P',
                'DTLIMITEWFLOW'   => $date->format('d/m/Y H:i:s'),
                '__is_new'        => true
            );
            array_push($dataSet, $row);
        }
        $newDataSet = new DataSet('operworkflow', $dataSet); 
        $newRequest = new DTO\Request\DataSet($newDataSet, "", "", ""); 
        $rowsComDefaults = $this->crudService->getRowsToSave($newRequest, 'operworkflow');
        $this->crudService->save('operworkflow', $rowsComDefaults);
        
    }
    
    private function operadoresEnvolvidoUnidade($nrprocessoaprova, $nritemprocaprova){
        $filtro = array(
            array(
                'columnName' => 'NRPROCESSOAPROVA',
                'operator' => '=',
                'value' => $nrprocessoaprova
            ),
            array(
                'columnName' => 'NRITEMPROCAPROVA',
                'operator' => '=',
                'value' => $nritemprocaprova
            )
        );
        $dataSourceName = 'operitempaprova';
        $this->parameterBagColumns = array();
        $filterCriteria= new FilterCriteria($dataSourceName, $filtro);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, array());
        $dataSetOper = $this->dataSourceManager->findBy($newFilterCriteria);
        $operEnvolvidos = $dataSetOper->getRows();
        if(empty($operEnvolvidos)){
            return [];
        } else {
            return $operEnvolvidos;
        }
    }
    
    public function editaVariacao(DTO\Request $request, DTO\Response $response){
        try{
            $this->connection->beginTransaction();
            $this->saveCrudAlteracoes($request, $response);
            $rows = $this->getRowsFromRequest($request);
            $this->verificaAlteracoes($request,$response,$rows);
            $this->connection->commit();
        } catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
            $this->connection->rollback();
        }
    }
    
    public function saveCrudAlteracoes(DTO\Request $request, DTO\Response $response) {
		$dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
		$rows = $this->crudService->getRowsToSave($request, $dataSourceName);
		foreach($rows as &$row){
		    if(isset($row['OPERADOR_TO_SAVE'])){
		        $row['CDOPERADOR'] = $row['OPERADOR_TO_SAVE'];
		    }
		}
		$this->crudService->save($dataSourceName, $rows);
        $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
	}
    
    public function VerificaAlteracoesDelete(DTO\Request $request, DTO\Response $response){
        try {
            $this->connection->beginTransaction();
            $this->deleteCrudAlteracoes($request, $response);
            $rows = $request->getDataSet()->getRows();
            foreach($rows as $row) {    
                $pkCDRECEITA = $row['CDRECEITA'];
                $rowReceita = array(
                    'CDRECEITA' => $pkCDRECEITA,
                    '__is_new' => false
                );
                $this->salvaAlteracoes($request,$response, $rowReceita, 'receitam');
            }
            $this->connection->commit();
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
            $this->connection->rollback();
        }
    }
	
	public function deleteCrudAlteracoes(DTO\Request $request, DTO\Response $response) {
		$request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
        $dataSet = $request->getDataSet();
        $dataSet->setDataSourceName($this->dataSourceName);
        $this->crudService->delete($response, $dataSet);
	}
    
    public function verificaAlteracoes(DTO\Request $request, DTO\Response $response, $rows){
        
        foreach($rows as $row) { 

            $pkCDRECEITA = $row['CDRECEITA'];
  
            $pkNRVARIACAOREC = $row['NRVARIACAOREC'];
            
            $rowReceita = array(
                'CDRECEITA' => $pkCDRECEITA,
                '__is_new' => false
            );
            $this->salvaAlteracoes($request,$response, $rowReceita, 'receitam');
            $rowVariacao = array(
                'CDRECEITA' => $pkCDRECEITA,
                'NRVARIACAOREC' => $pkNRVARIACAOREC,
                '__is_new' => false
            );
            $this->salvaAlteracoes($request,$response, $rowVariacao, 'variacaorec');
        }
    }
    
    private function salvaAlteracoes(DTO\Request $request, DTO\Response $response, $row, $dataSourceName){
        
        $dataAtual = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
        $operador = $this->environment->getCdOperador();
        $row['DTULTATU'] = $dataAtual;
        $row['DSOPERULTATU'] = $operador;
    
        $method = $request->getMethod();
        $routePath = $request->getRoutePath();
        $userId = $request->getUserId();
        
        $newRequest = new DTO\Request\Row($row, $method, $routePath, $userId);
        $rows = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
		$this->crudService->save($dataSourceName, $rows);
        $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
    }
    
}