<?php

namespace Teknisa\COC\Services;

use Teknisa\GER\Utilities\Environment;
use Teknisa\GER\Services\CrudService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class ReceitaService {

    const ESCOPO_VALIDACAO_REFERENCIA_RECEITA = array(
        "QUERY_TABLE"           => array(
            "UTILRECFLEX"       => "SELECT UT.CDFILIAL, FL.NMFILIAL, UT.CDRECEITA, UT.NRSEQUTILRECFLEX, UT.DTINIVIGENCIA, UT.DTFIMVIGENCIA, UT.NRORG, 'APP-00162' AS CODEERROR FROM UTILRECFLEX UT JOIN FILIAL FL ON FL.CDFILIAL = UT.CDFILIAL AND FL.NRORG = UT.NRORG WHERE UT.CDRECEITA = :CDRECEITA AND UT.NRORG = :NRORG",
            "SUBRECEITAM"       => "SELECT SR.CDRECEITA, RC.DSRECEITA, SR.CDSUBRECEITA, SR.NRORG, 'APP-00163' AS CODEERROR FROM SUBRECEITAM SR INNER JOIN RECEITAM RC ON RC.CDRECEITA = SR.CDRECEITA WHERE SR.CDSUBRECEITA = :CDRECEITA AND SR.NRORG = :NRORG",
            "RECSUBGRUM"        => "SELECT RSG.CDRECEITA, RSG.CDSUBGRU, SG.DSSUBGRU, RSG.NRORG, 'APP-00164' AS CODEERROR FROM RECSUBGRUM RSG INNER JOIN SUBGRUPOM SG ON SG.CDSUBGRU = RSG.CDSUBGRU AND SG.NRORG = RSG.NRORG WHERE RSG.CDRECEITA = :CDRECEITA AND RSG.NRORG = :NRORG",
            "PADRAOPRODREC"     => "SELECT PD.CDRECEITA, PD.NRPADRAOPRODREC, PD.CDFILIAL, FL.NMFILIAL, PD.NRORG, 'APP-00165' AS CODEERROR FROM PADRAOPRODREC PD INNER JOIN FILIAL FL ON FL.CDFILIAL = PD.CDFILIAL AND FL.NRORG = PD.NRORG WHERE PD.CDRECEITA = :CDRECEITA AND PD.NRORG = :NRORG",
            "FLUXPRODREC"       => "SELECT FP.CDRECEITA, FP.NRFLUXPRODREC, FP.CDFILIAL, FL.NMFILIAL, FP.CDSETOR, ST.SGSETOR, FP.NRCATEGORIAPROD, CTP.NMCATEGORIAPROD, FP.NRORG, 'APP-00166' AS CODEERROR FROM FLUXPRODREC FP INNER JOIN FILIAL FL ON FL.CDFILIAL = FP.CDFILIAL AND FL.NRORG = FP.NRORG INNER JOIN SETOR ST ON ST.CDSETOR = FP.CDSETOR AND ST.NRORG = FP.NRORG INNER JOIN CATEGORIAPROD CTP ON CTP.NRCATEGORIAPROD = FP.NRCATEGORIAPROD AND CTP.NRORG = FP.NRORG WHERE FP.CDRECEITA = :CDRECEITA AND FP.NRORG = :NRORG",
            "AUXRECSUBSEMAM"    => "SELECT ACS.CDFILIAL, ACS.CDCIA, ACS.CDMANCIA, ACS.NRSERVICOCIAFILI, ACS.CDCLASSE, ACS.CDSUBGRU, ACS.NRSEQSUBGRUSEMAM, ACS.CDCICLO, ACS.CDRECEITA, ACS.NRORG, FL.NMFILIAL, CIA.SGCIA, MN.DSMANUAL, SCF.DSSERVICOCIAFILI, CL.SGCLASSE, SG.DSSUBGRU, CC.IDCICLO, 'APP-00167' AS CODEERROR FROM AUXRECSUBSEMAM ACS INNER JOIN FILIAL FL ON FL.CDFILIAL = ACS.CDFILIAL AND FL.NRORG = ACS.NRORG INNER JOIN COMPANHIA CIA ON CIA.CDCIA = ACS.CDCIA AND CIA.NRORG = ACS.NRORG INNER JOIN MANUALCIAM MN ON MN.CDFILIAL = ACS.CDFILIAL AND MN.CDCIA = ACS.CDCIA AND MN.CDMANCIA = ACS.CDMANCIA AND MN.NRORG = ACS.NRORG INNER JOIN SERVICOCIAFILI SCF ON SCF.CDFILIAL = ACS.CDFILIAL AND SCF.CDCIA = ACS.CDCIA AND SCF.NRSERVICOCIAFILI = ACS.NRSERVICOCIAFILI AND SCF.NRORG = ACS.NRORG INNER JOIN CLASSE CL ON CL.CDCLASSE = ACS.CDCLASSE AND CL.NRORG = ACS.NRORG INNER JOIN SUBGRUPOM SG ON SG.CDSUBGRU = ACS.CDSUBGRU AND SG.NRORG = ACS.NRORG INNER JOIN CICLO CC ON CC.CDCICLO = ACS.CDCICLO AND CC.NRORG = ACS.NRORG WHERE ACS.CDRECEITA = :CDRECEITA AND ACS.NRORG = :NRORG",
            //"EDITSUBSCOMPMG"    => "SELECT ESC.NREDITSUBSCOMPMG, ESC.NRSUBSCOMPMG, ESC.IDTIPOACAO, SCM.DSSUBSCOMPMG, SCM.IDSITUACAO, SCM.DTEFETIVACAO, ESC.NRORG, 'APP-00168' AS CODEERROR FROM EDITSUBSCOMPMG ESC INNER JOIN SUBSCOMPMG SCM ON SCM.NRSUBSCOMPMG = ESC.NRSUBSCOMPMG AND SCM.NRORG = ESC.NRORG WHERE ESC.CDRECEITA = :CDRECEITA AND ESC.NRORG = :NRORG",
            "EDITSUBSCOMPMENU"  => "SELECT ESC.NREDITSUBSCM, ESC.NRSUBSCOMPMENU, ESC.IDTIPOACAO, SCM.DSSUBSCOMPMENU, SCM.IDSITUACAO, SCM.DTEFETIVACAO, ESC.NRORG, 'APP-00169' AS CODEERROR FROM EDITSUBSCOMPMENU ESC INNER JOIN SUBSCOMPMENU SCM ON SCM.NRSUBSCOMPMENU = ESC.NRSUBSCOMPMENU AND SCM.NRORG = ESC.NRORG WHERE ESC.CDRECEITA = :CDRECEITA AND ESC.NRORG = :NRORG",
            "RECCKLISTPRODINF"  => "SELECT CK.NRCKLISTPRODINF, CK.CDFILIAL, CK.CDCKLISTPRODINF, CK.DTPRODUCAOINF, CK.IDTIPOPLAN, RCK.CDRECEITA, RCK.NRORG, 'APP-00170' AS CODEERROR FROM RECCKLISTPRODINF RCK INNER JOIN CKLISTPRODINF CK ON CK.NRCKLISTPRODINF = RCK.NRCKLISTPRODINF AND CK.CDFILIAL = RCK.CDFILIAL AND CK.NRORG = RCK.NRORG WHERE RCK.CDRECEITA = :CDRECEITA AND RCK.NRORG = :NRORG",
            "RECSOLEMERGPROD"   => "SELECT CK.NRSOLEMERGPROD, CK.CDFILIAL, RCK.NRRECSOLEMERGPROD, CK.DTPRODUCAO, CK.IDSITUACAO, RCK.CDRECEITA, RCK.NRORG, 'APP-00171' AS CODEERROR FROM RECSOLEMERGPROD RCK INNER JOIN SOLEMERGPROD CK ON CK.NRSOLEMERGPROD = RCK.NRSOLEMERGPROD AND CK.CDFILIAL = RCK.CDFILIAL AND CK.NRORG = RCK.NRORG WHERE RCK.CDRECEITA = :CDRECEITA AND RCK.NRORG = :NRORG"
        ),
        "PARAMS"                => array('CDRECEITA', 'NRORG')
    );

    const ESCOPO_DELETA_TABELAS_RECEITA = array(
        "TABLES"       => array(
            "TESTERECEITAM", 
            "FILIRECEITA",   
            "PRORECEITAM",
            "SUBRECEITAM",    
            "MODPREPRECM",   
            "VARIACAOREC",   
            "CATEGORIAREC"  
        ),
        "PARAMS"            => array('CDRECEITA', 'NRORG'),
        "QUERY"             => 'DELETE FROM $param WHERE CDRECEITA = :CDRECEITA AND NRORG = :NRORG'
    );


	protected $connection;
	protected $crudService;
    protected $messagesInflightor;

	public function __construct($connection, $MessagesProcedureInflightor, $crudService) {
        $this->connection           = $connection;
        $this->messagesInflightor   = $MessagesProcedureInflightor;
		$this->crudService          = $crudService;
    }
    
    public function getScopeTableReferencia(){
        return self::ESCOPO_VALIDACAO_REFERENCIA_RECEITA['QUERY_TABLE'];
    }

    public function getScopeParamsReferencia(){
        return self::ESCOPO_VALIDACAO_REFERENCIA_RECEITA['PARAMS'];
    }

    public function getScopeTableDelete(){
        return self::ESCOPO_DELETA_TABELAS_RECEITA['TABLES'];
    }

    public function getScopeQueryDelete(){
        return self::ESCOPO_DELETA_TABELAS_RECEITA['QUERY'];
    }

    public function getScopeParamsDelete(){
        return self::ESCOPO_DELETA_TABELAS_RECEITA['PARAMS'];
    }

	public function executeDeleteOrInsert($sql, $params){
		$query = $this->connection->prepare($sql);
		return $query->execute($params);
    }

    public function findScopeReferenciaReceita($row){
        $tableScope = $this->getScopeTableReferencia();
        $params = $this->_buildParamsByRow($row, $this->getScopeParamsReferencia());
        $callbackScope = array();
        foreach($tableScope as $tableName => $query){
            $resultQuery = $this->executeSelect($query, $params);
            if(!empty($resultQuery)){
                $callbackScope[$tableName] = $resultQuery;
            }
        }
        return $callbackScope;
    }

    public function deleteScopeReceita($row){
        $tableScope = $this->getScopeTableDelete();
        $params = $this->_buildParamsByRow($row, $this->getScopeParamsDelete());
        $callbackScope = array();
        foreach($tableScope as $tableScope){
            $query = $this->getScopeQueryDelete();
            $queryTable = str_replace('$param', $tableScope, $query);
            $callbackScope[$tableScope] = $this->executeDeleteOrInsert($queryTable, $params);
        }
        return $callbackScope;
    }

    public function convertMessage($arrayError){
        $decodeMsg = array();
        foreach($arrayError as $errorTable){
            foreach($errorTable as $error){
                $arrIndex = array();
                foreach(array_keys($error) as $key){
                    $arrIndex [] = $error[$key];
                }
                array_push($decodeMsg, $this->messagesInflightor->getMessage($error['CODEERROR'], $arrIndex));
            }
        }
        return $decodeMsg;
    }

    private function executeSelect($sql, $params) {
	    $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
        return $stmt->fetchAll(\PDO::FETCH_ASSOC);
    }
    
    private function _buildParamsByRow($values, $params){
        $newFilter = array();
    	foreach ($params as $key) {
    		if (gettype($key) != 'integer') {
    			$newFilter[$key] = $values[$key];
    		}
    	}
    	return $newFilter;
    }
}